
// Asounding Audio Engine for iOS
// Copyright (c) 2011  Michel Fortin
//
// You must obtain a comercial license for using this code in software
// distributed not under the GNU GPL. See <http://michelf.com/asounding/>.
//
// This program is free software: you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free 
// Software Foundation, either version 3 of the License, or (at your option) 
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT 
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
// more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program. If not, see <http://www.gnu.org/licenses/>.
//
// As a special exception to the GPL, you may distribute a compiled 
// application including this file on the Apple App Store for iOS without 
// having to redistribute the digital keys needed to reproduce the digital 
// signature needed to run an application on an iOS device. This exception 
// does not invalidate any other requirements put on you by the GPL.

#import <Foundation/Foundation.h>
#import <OpenAL/al.h>
#import <OpenAL/alc.h>


@class AVAudioPlayer, ASNMusic, ASNSource;

/*!
 @abstract   Engine class handling music and and system events.
 @discussion The ASNEngine singleton is in charge of managing the audio state 
 of the application. It stops and restarts the background music and sounds when
 the application is deactivated and then reactivated.
 
 ASNEngine is also the application's background music player, allowing you to
 set the currently playing music, and do fade outs. Special care has been taken
 so that music playback is completely asynchrounous: starting playback or 
 fading out won't cause the calling thread to block while.
 */
@interface ASNEngine : NSObject
{
	ALCcontext*				context;
	ALCdevice*				device;

	BOOL					isAudioPlaybackSuspended;
	BOOL					isApplicationActive;
	BOOL					isAudioSessionInterrupted;
	BOOL					isMusicEnabled;
	BOOL					isSoundsEnabled;
	BOOL					isAudioEnabled;
	
	BOOL					iPodPlaybackDisabled;
	UInt32					iPodIsPlaying;
	
	NSMutableSet*			sources;
	
	ASNMusic*				music;
	float					musicVolume;
	NSThread*				musicControlThread;
	NSCondition*			musicControlCondition;
	int						musicControlState;
	float					musicFadeStep;
}

/*! @group  Getting the Engine  */
/*!
 @abstract   Retrieve the shared engine instance.
 @discussion A shared instance of ASNEngine is created on the first call to 
 this function.
 */
+ (ASNEngine *)sharedEngine;


/*! @group  Enabling Audio Playback  */
/*!
 @abstract   Master switch for disabling all audio playback (sound sources 
 and music).
 @discussion Setting this to NO will suspend all audio output from Asounding: 
 background music and currently running sounds effects are paused until 
 audioEnabled is set again to YES.
 
 Default value is YES.
 */
@property (assign, getter=isAudioEnabled) BOOL audioEnabled;

/*!
 @abstract   Independent control to disable sounds.
 @discussion Setting this to NO will prevent stop currently playing sound 
 sources and prevent sound sources from playing until set to YES again.
 
 Default value is YES.
 */
@property (assign, getter=isSoundsEnabled) BOOL soundsEnabled;

/*!
 @abstract   Independent control to disable music alone.
 @discussion Setting this to NO will disable music playback, pausing any
 currently playing music and make any call to playMusic wait until music is
 enabled again before starting playback.
 
 Default value is YES.
 */
@property (getter=isMusicEnabled) BOOL musicEnabled;

/*!
 @abstract   Whether iPod is allowed to play while this application is active.
 @discussion Setting this to YES will force the iPod to stop playing music and 
 allows Asounding to play its background music. Setting it to NO may block 
 for a second while the iPod stops playing.
 
 Default value is NO, which disables the background music when the iPod is
 playing.
 */
@property (assign, getter=isIPodPlaybackDisabled) BOOL iPodPlaybackDisabled;


/*! @group  Checking Audio State  */
/*!
 @abstract   Tell whether the iPod is playing music or not.
 @discussion This property is updated only when audio playback is suspended, 
 which happens when you manually disable it or because of a system event (like
 the device going to sleep).
 */
@property (readonly, nonatomic, assign)	UInt32 iPodIsPlaying;

/*!
 @abstract   Tell whether the audio session has been interrupted because of 
 another audio session on the system.
 */
@property (readonly, nonatomic, assign)	BOOL audioSessionInterrupted;

/*!
 @abstract   Tell if Asounding audio playback is currently suspended.
 @discussion Audio playback is suspended when the audioEnabled property is set 
 to NO, when the application is deactivated (such as when the device is going 
 to sleep) or when the audio session gets interrupted.
 */
@property (readonly) BOOL audioPlaybackSuspended;


/*! @group  Controlling Music Playback  */
/*!
 @abstract   Tell if background music is currently playing.
 @discussion YES if background music from the Engine is currently playing, 
 NO if not.
 */
@property (readonly, getter=isMusicPlaying) BOOL musicPlaying;


/*! @group  Controlling Music Playback  */
/*!
 @abstract   Tell if background music is currently paused.
 @discussion YES if background music from the Engine has been paused and not yet resumed, NO if not.
 */
@property (readonly, getter=isMusicPaused) BOOL musicPaused;

/*!
 @abstract   The current background music.
 @discussion This property contains the the music to be played in background.
 Changing its value will stop the current background music (if one was playing)
 and preload the new music.
 */
@property (retain) ASNMusic * music;

/*!
 @abstract   The playback volume for the background music.
 @discussion Changing this property will affect the volume at which the
 background music is played. Volume should be a float between 0.0 and 1.0. By 
 default, this property is set to 0.5.
 */
@property float musicVolume;

/*!
 @abstract   Start playing the current background music.
 @discussion This method starts the backgrond music immediately if there is no
 audio interruption and music is enabled, or else schedule the music to be started
 once music is enabled and audio interruptions are lifted.
 
 Music will not start if the iPod is currently playing music, unless the 
 shouldMusicPlayOverIPod property is set to YES.
 
 If the background music was previously paused, it restarts from the position
 it was paused, othewise it starts from the beginning.
 */
- (void)playMusic;

/*!
 @abstract   Pause the current background music; keep current position.
 @discussion This method stops the backgrond music, but keep the current 
 position so that it can be restarted from here.
 */
- (void)pauseMusic;

/*!
 @abstract   Stop the current background music; rewind to beginning.
 @discussion This method stops the backgrond music and rewind the current 
 position to the beginning.
 */
- (void)stopMusic;

/*!
 @abstract   Fade out and stop the current background music.
 @param      seconds  The number of seconds for the fade out effect.
 @discussion This method lowers the volume of the backgrond music progressively
 and then stops it, rewinding to the beginning. The time taken by the fade out
 effect is specified in seconds.
 */
- (void)fadeMusicWithin:(NSTimeInterval)seconds;


/*! @group  Registering Sound Sources  */
/*!
 @abstract   The list of sound sources registered with the engine.
 @discussion Sound sources when created automatically register themselves with
 the engine. This property lists all the sound sources currently in existance.
 */
@property (readonly) NSSet * sources;

/*!
 @abstract   Add a sound source to the engine.
 @param      newSource  The sound source to add.
 @discussion This method should not be called directly. It is called by 
 ASNSource during initialization.
 */
- (void)addSource:(ASNSource *)newSource;

/*!
 @abstract   Remove a sound source from the engine. Should not be called directly.
 @param      theSource  The sound source to remove.
 @discussion This method should not be called directly. It is called when
 by ASNSource before deallocation.
 */
- (void)removeSource:(ASNSource *)theSource;

@end
