
// Asounding Audio Engine for iOS
// Copyright (c) 2011  Michel Fortin
//
// You must obtain a comercial license for using this code in software
// distributed not under the GNU GPL. See <http://michelf.com/asounding/>.
//
// This program is free software: you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free 
// Software Foundation, either version 3 of the License, or (at your option) 
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT 
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
// more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program. If not, see <http://www.gnu.org/licenses/>.
//
// As a special exception to the GPL, you may distribute a compiled 
// application including this file on the Apple App Store for iOS without 
// having to redistribute the digital keys needed to reproduce the digital 
// signature needed to run an application on an iOS device. This exception 
// does not invalidate any other requirements put on you by the GPL.

#import <Foundation/Foundation.h>
#import <OpenAL/al.h>

@class ASNSound, ASNEngine;


/*!
 @abstract   A sound source capable of emitting sounds.
 @discussion Each source is capable of playing one sound at a time. To play
 a sound, you set sound property to the ASNSound object you want, then control
 the playback with the play, pause, stop and rewind methods.
 */
@interface ASNSource : NSObject
{
	ASNEngine*	engine;
	ASNSound*	sound;
	ALuint		source;
	ALboolean	looping;
	ALfloat		pitch;
}

/*! @group  Creating a Source */
/*!
 @abstract   Initialize a new sound source.
 @discussion After initialization, you can attatch a sound to the source and 
 start playing it.
 */
- (id)init;


/*! @group  Controlling Sound Playback */
/*!
 @abstract   Start playing current sound.
 @discussion The sound starts playing at the current position.
 */
- (void)play;
/*!
 @abstract   Stop playing current sound; rewinding at the start of the sound.
 */
- (void)stop;
/*!
 @abstract   Start playing current sound; keeping current position for next
 */
- (void)pause;
/*!
 @abstract   Set the current position back to the start of the sound.
 */
- (void)rewind;

/*!
 @abstract   The looping state for this source.
 @discussion When this property is set to true, the sound will restart as soon
 it reaches the end.
 */
@property BOOL looping;

- (BOOL)looping;
- (void)setLooping:(BOOL)flag;

/*!
 @abstract   The pitch for this source.
 @discussion This property changes the playback speed of the source.
 */
@property ALfloat pitch;

- (ALfloat)pitch;
- (void)setPitch:(ALfloat)aPitch;

/*!
 @abstract   The sound to play from this source.
 @discussion This is the sound that will play when calling the various playback
 methods on this source.
 
 Setting the sound will stop playback of this source if it is currently playing.
 
 The same ASNSound object can be used by multiple sources at the same time.
 */
@property(retain) ASNSound *sound;

- (ASNSound *)sound;
- (void)setSound:(ASNSound *)aSound;

@property(assign) ASNEngine *engine;

@end
